@@ -1,29 +1,88 @@
 package JSON::Karabiner::Manipulator ;
-$JSON::Karabiner::Manipulator::VERSION = '0.010';
+$JSON::Karabiner::Manipulator::VERSION = '0.011';
 use strict;
 use warnings;
 use Carp;
-use Exporter 'import';
-our @EXPORT = 'new_manipulator';
+use Exporter;
+our @EXPORT = qw'new_manipulator add_action add_description add_condition add_parameter add_key_code
+  add_key_code add_any add_optional_modifiers add_mandatory_modifiers add_simultaneous add_simultaneous_options add_consumer_key_code add_pointing_button add_shell_command add_select_input_source add_set_variable add_mouse_key add_modifiers
+  add_identifier add_description add_value add_bundle_identifiers add_file_path add_input_source add_keyboard_types add_variable add_description _dump_json _fake_write_file write_file';
+
+sub import {
+  strict->import;
+  warnings->import;
+  goto &Exporter::import
+}
 
 sub new_manipulator {
   my $class = 'JSON::Karabiner::Manipulator';
   shift if $_[0] =~ /^JSON::Karabiner::Manipulator$/;
 
   my @kb_obj_args = @_;
-  use Data::Dumper qw(Dumper);
   my $self = {
     actions => {},
     _disable_validity_tests => 0,
     _kb_obj_args => \@kb_obj_args,
+    _fake_write_flag => 0,
   };
   bless $self, $class;
+  {
+    no warnings 'once';
+    $main::current_manip = $self;
+  }
   return $self;
 }
 
-sub add_action {
+sub AUTOLOAD {
+  our $AUTOLOAD;
+  use Data::Dumper qw(Dumper);
+  my $program = $AUTOLOAD;
+  my ($func) = $program =~ /.*::(.*)$/;
+  my @action_functions = qw (add_key_code add_any add_optional_modifiers add_mandatory_modifiers add_simultaneous add_simultaneous_options add_consumer_key_code add_pointing_button add_shell_command add_select_input_source add_set_variable add_mouse_key add_modifiers);
+
+  my $is_action_function = grep { $_ eq $func } @action_functions;
+  if ($is_action_function) {
+    my $current_action;
+    {
+      no warnings 'once';
+      $current_action = $main::current_action;
+    }
+    $current_action->$func(@_);
+    return;
+  }
+
+  my @condition_functions = qw(add_identifier add_description add_value add_bundle_identifiers add_file_path add_input_source add_keyboard_types add_variable add_description);
+
+  my $is_condition_function = grep { $_ eq $func } @condition_functions;
+  if ($is_condition_function) {
+    my $current_condition;
+    {
+      no warnings 'once';
+      $current_condition = $main::current_condition;
+    }
+    $current_condition->$func(@_);
+    return;
+  }
+
+}
+
+sub _get_args {
+  my @args = @_;
+  croak "No args passed" unless @args;
+
   my $s = shift;
-  my $type = shift;
+  my $type;
+  if (ref $s) {
+    $type = shift;
+  } else {
+    $type = $s;
+    $s = $main::current_manip;
+  }
+  return ($s, $type, @_);
+}
+
+sub add_action {
+  my ($s, $type) = _get_args(@_);
   croak 'To add a action, you must tell me which kind you\'d like to add' if !$type;
 
   my $uctype = ucfirst($type);
@@ -38,8 +97,7 @@ sub add_action {
 }
 
 sub add_condition {
-  my $s = shift;
-  my $type = shift;
+  my ($s, $type) = _get_args(@_);
   croak 'To add a condition, you must tell me which kind you\'d like to add' if !$type;
 
   my $uctype = ucfirst($type);
@@ -55,17 +113,14 @@ sub add_condition {
 }
 
 sub add_description {
-  my $s = shift;
-  my $desc = shift;
+  my ($s, $desc) = _get_args(@_);
 
   croak 'To add a description, you must provide one' if !$desc;
   $s->{actions}{description} = $desc;
 }
 
 sub add_parameter {
-  my $s = shift;
-  my $param = shift;
-  my $value = shift;
+  my ($s, $param, $value) = _get_args(@_);
   croak 'To add a parameter, you must tell me which kind you\'d like to add' if !$param;
   croak 'To add a parameter, you must provide a value' if !$value;
 
@@ -112,7 +167,6 @@ sub TO_JSON {
 }
 
 sub _do_validity_checks {
-  use Data::Dumper qw(Dumper);
   my $s = shift;
   my $actions = shift;
   my $from = $actions->{from};
@@ -120,26 +174,69 @@ sub _do_validity_checks {
 }
 
 sub _dump_json {
-  use Data::Dumper qw(Dumper);
-
-  my $s = shift;
-  print Dumper \@_;
+  my $s = $main::current_manip;
   my @kb_obj_args = @{$s->{_kb_obj_args}};
-  print Dumper \@kb_obj_args;
   if (!@kb_obj_args) {
     croak "The _dump_json method cannot be run on this manipulator.";
   }
 
   require JSON::Karabiner;
+  my $little_title = shift @kb_obj_args;
+  unshift @kb_obj_args, 'SET WITH write_file METHOD';
   my $kb_obj = JSON::Karabiner->new( @kb_obj_args );
-  my $rule = $kb_obj->add_rule('fake rule name');
+
+  my $rule = $kb_obj->add_rule($little_title);
   my $temp_manip = $rule->add_manipulator();
   %{$temp_manip} = %{$s};
   $kb_obj->_dump_json;
 }
 
+sub write_file {
+  my $s = $main::current_manip;
+  my $title = shift;
+  my @kb_obj_args = @{$s->{_kb_obj_args}};
+  if (!@kb_obj_args) {
+    croak "The _write_file method cannot be run on this manipulator.";
+  }
 
+  croak 'You must supply a title for the first manipulator' if !$title && !$main::file_written;
+  if (!$title) {
+    $title = $main::file_title_written;
+  }
 
+  require JSON::Karabiner;
+  my $little_title = shift @kb_obj_args;
+  unshift @kb_obj_args, $title;
+  my $kb_obj = JSON::Karabiner->new( @kb_obj_args );
+  if ($s->{_fake_write_flag}) {
+    $kb_obj->{_fake_write_flag} = 1;
+  }
+  my $rule = $kb_obj->add_rule($little_title);
+  my $temp_manip = $rule->add_manipulator();
+  %{$temp_manip} = %{$s};
+  $kb_obj->write_file(1);
+  $kb_obj->{_fake_write_flag} = 0;
+  {
+    no warnings 'once';
+    $main::file_written = $kb_obj_args[1];
+    $main::file_title_written = $kb_obj_args[0];
+  }
+
+}
+
+sub _fake_write_file {
+  my $s = $main::current_manip;
+  my $title = shift;
+
+  my @kb_obj_args = @{$s->{_kb_obj_args}};
+  if (!@kb_obj_args) {
+    croak "The _fake_write method cannot be run on this manipulator.";
+  }
+
+  $s->{_fake_write_flag} = 1;
+  $s->write_file($title);
+  $s->{_fake_write_flag} = 0;
+}
 
 sub _do_from_validity_checks {
   my $s = shift;
@@ -193,7 +290,7 @@ Methods are listed below for reference purposes only.
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 AUTHOR
 
