@@ -1,5 +1,5 @@
 package JSON::Karabiner::Manipulator::Actions ;
-$JSON::Karabiner::Manipulator::Actions::VERSION = '0.010';
+$JSON::Karabiner::Manipulator::Actions::VERSION = '0.011';
 use strict;
 use warnings;
 use JSON;
@@ -19,6 +19,10 @@ sub new {
 
   };
   bless $self, $class;
+  {
+    no warnings 'once';
+    $main::current_action = $self;
+  }
   return $self;
 }
 
@@ -82,7 +86,7 @@ for reference purposes only.
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 AUTHOR
 
