@@ -1,5 +1,5 @@
 package JSON::Karabiner::Manipulator::Conditions ;
-$JSON::Karabiner::Manipulator::Conditions::VERSION = '0.010';
+$JSON::Karabiner::Manipulator::Conditions::VERSION = '0.011';
 use strict;
 use warnings;
 use JSON;
@@ -15,12 +15,15 @@ sub new {
 
   };
   bless $self, $class;
+  {
+    no warnings 'once';
+    $main::current_condition = $self;
+  }
   return $self;
 }
 
 sub TO_JSON {
   my $obj = shift;
-  use Data::Dumper qw(Dumper);
   my $name = $obj->{def_name};
   my $value = $obj->{data};
   my @data_hash = @{$obj->{data}};
@@ -112,7 +115,7 @@ See L<Karabiner official documentation|https://karabiner-elements.pqrs.org/docs/
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 AUTHOR
 
