@@ -1,5 +1,5 @@
 package JSON::Karabiner::Manipulator::Conditions::Variable_if ;
-$JSON::Karabiner::Manipulator::Conditions::Variable_if::VERSION = '0.010';
+$JSON::Karabiner::Manipulator::Conditions::Variable_if::VERSION = '0.011';
 use strict;
 use warnings;
 use JSON;
@@ -36,10 +36,8 @@ sub add_description {
 
 sub TO_JSON {
   my $obj = shift;
-  use Data::Dumper qw(Dumper);
   my $name = $obj->{def_name};
   my $value = $obj->{data};
-  print Dumper $value;
   my %super_hash = (%$value, type => $name);
   return { %super_hash };
 
@@ -68,7 +66,7 @@ JSON::Karabiner::Manipulator::Conditions::Variable_if - definition for Frontmost
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 AUTHOR
 
